# -*- coding: utf-8 -*-
# Copyright 2015 grafana-dashboard-builder contributors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from grafana_dashboards.components.base import JsonListGenerator, JsonGenerator

__author__ = 'Jakub Plichta <jakub.plichta@gmail.com>'


class Panels(JsonListGenerator):
    def __init__(self, data, registry):
        super(Panels, self).__init__(data, registry, PanelsItemBase)


class PanelsItemBase(JsonGenerator):
    pass


class Graph(PanelsItemBase):
    def gen_json_from_data(self, data, context):
        panel_json = {
            'type': 'graph',
            'title': self.data.get('title', None),
            'span': self.data.get('span', None),
            'targets': [
                {
                    'target': self.data['target']
                }
            ]
        }
        if 'y_formats' in self.data:
            panel_json['y_formats'] = self.data['y_formats']
        if 'grid' in self.data:
            panel_json['grid'] = {
                'leftMax': self.data['grid'].get('leftMax', None),
                'rightMax': self.data['grid'].get('rightMax', None),
                'leftMin': self.data['grid'].get('leftMin', None),
                'rightMin': self.data['grid'].get('rightMin', None)
            }
        return panel_json



# -*- coding: utf-8 -*-
# Copyright 2015 grafana-dashboard-builder contributors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from grafana_dashboards.components.base import JsonListGenerator, JsonGenerator

__author__ = 'Jakub Plichta <jakub.plichta@gmail.com>'


class Templates(JsonListGenerator):
    def __init__(self, data, registry):
        super(Templates, self).__init__(data, registry, TemplatesItemBase)


class TemplatesItemBase(JsonGenerator):
    pass


class Query(TemplatesItemBase):
    def gen_json_from_data(self, data, context):
        processed_parts = []
        queries = []
        if not self.data.get('query'):
            return queries
        for query_part in self.data['query'].split('.'):
            if query_part.startswith('$'):
                is_first = False if queries else True
                query = query_part[1:]
                metric = '*'
                template_json = {
                    'type': 'query',
                    'refresh_on_load': is_first,
                    'name': query,
                    'refresh': is_first
                }
                if query in self.data:
                    query_config = self.data[query]
                    metric = query_config.get('metric', metric)
                    if 'current' in query_config:
                        current = query_config['current']
                        template_json['current'] = {
                            'text': current,
                            'value': current
                        }
                    if 'options' in query_config:
                        template_json['options'] = [{'text': option, 'value': option} for option in
                                                    (query_config['options'])]
                template_json['query'] = '.'.join(processed_parts + [metric])
                queries.append(template_json)
            processed_parts.append(query_part)
        return queries



# -*- coding: utf-8 -*-
# Copyright 2015 grafana-dashboard-builder contributors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import os
import errno

__author__ = 'Jakub Plichta <jakub.plichta@gmail.com>'


class DashboardJSONEncoder(json.JSONEncoder):
    def __init__(self):
        super(DashboardJSONEncoder, self).__init__(sort_keys=True, indent=2, separators=(',', ': '))


class DashboardBuilder(object):
    def __init__(self, output_folder):
        super(DashboardBuilder, self).__init__()
        self.output_folder = output_folder
        if not os.path.exists(output_folder):
            os.makedirs(output_folder)
        if not os.path.isdir(output_folder):
            raise Exception('out must be a directory')

    def build_dashboards(self, project):
        """

        :type project: grafana_dashboards.components.Project
        """
        for context in project.get_contexts():
            for dashboard in project.get_dashboards():
                json_obj = dashboard.gen_json(context)
                dirname = os.path.join(self.output_folder, project.name)
                try:
                    os.makedirs(dirname)
                except OSError as e:
                    if e.errno != errno.EEXIST:
                        raise
                dashboard_path = os.path.join(dirname, context.expand_placeholders(dashboard.name) + '.json')
                with file(dashboard_path, 'w') as f:
                    json.dump(json_obj, f, sort_keys=True, indent=2, separators=(',', ': '))



# -*- coding: utf-8 -*-
# Copyright 2015 grafana-dashboard-builder contributors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from grafana_dashboards.components.base import JsonGenerator, get_component_type
from grafana_dashboards.components.rows import Rows
from grafana_dashboards.components.templates import Templates

__author__ = 'Jakub Plichta <jakub.plichta@gmail.com>'


class Dashboard(JsonGenerator):
    def gen_json_from_data(self, data, context):
        nav = {
            'type': 'timepicker'
        }
        json_data = {
            'title': data.get('title', self.name),
            'nav': [
                nav
            ]
        }
        if 'time' in data:
            json_data['time'] = {
                'from': data['time']['from'],
                'to': data['time']['to']
            }
        if 'tags' in data:
            json_data['tags'] = data.get('tags')
        if 'time_options' in data:
            nav['time_options'] = data.get('time_options', [])
        if 'refresh_intervals' in data:
            nav['refresh_intervals'] = data.get('refresh_intervals', [])
        if get_component_type(Rows) in data:
            json_data['rows'] = self.registry.create_component(Rows, data).gen_json()
        if get_component_type(Templates) in data:
            json_data['templating'] = {
                'list': self.registry.create_component(Templates, data).gen_json(),
                'enable': True
            }
        return json_data



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright 2015 grafana-dashboard-builder contributors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import imp
import logging
import os

from grafana_dashboards.builder import DashboardBuilder
from grafana_dashboards.parser import DefinitionParser

__author__ = 'Jakub Plichta <jakub.plichta@gmail.com>'


def main():
    logging.basicConfig(format='%(asctime)s %(levelname)s %(message)s', level=logging.DEBUG)
    parser = argparse.ArgumentParser(conflict_handler="resolve")
    parser.add_argument('-p', '--path', required=True, nargs='+', type=str,
                        help='List of path to YAML definition files')
    parser.add_argument('--project',
                        help='(deprecated) Location of the file containing project definition.')
    parser.add_argument('-o', '--out',
                        help='Path to output folder')
    parser.add_argument('--plugins', nargs='+', type=str,
                        help='List of external component plugins to load')
    args = parser.parse_args()

    if args.plugins:
        for plugin in args.plugins:
            try:
                imp.load_source('grafana_dashboards.components.$loaded', plugin)
            except Exception as e:
                print 'Cannot load plugin %s: %s' % (plugin, str(e))

    paths = []
    if args.project:
        args.path.append(args.project)
    for path in args.path:
        if os.path.isdir(path):
            for root, dirs, filenames in os.walk(path):
                paths += [os.path.join(root, filename) for filename in filenames]
        else:
            paths.append(path)
    projects = DefinitionParser().load_projects(paths)
    builder = DashboardBuilder(args.out)
    for project in projects:
        builder.build_dashboards(project)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# Copyright 2015 grafana-dashboard-builder contributors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import yaml

from grafana_dashboards.components.base import ComponentRegistry
from grafana_dashboards.components.projects import Project

__author__ = 'Jakub Plichta <jakub.plichta@gmail.com>'


class DefinitionParser(object):
    def __init__(self):
        super(DefinitionParser, self).__init__()

    def load_projects(self, paths):
        registry = ComponentRegistry()
        for path in paths:
            with file(path, 'r') as fp:
                for component in self._iter_over_all(yaml.load_all(fp)):
                    registry.add(component)
        return registry[Project]

    @staticmethod
    def _iter_over_all(documents):
        return (component
                for document in documents
                for component in document)
